#include "f81866a.h"

static short OLD_LDN;
static short CUR_LDN;
static bool HAVE_PERMS = false;
static bool LPC_OPEN = false;

#ifdef __MINGW32__
typedef void(__stdcall *lpOut32)(short, short);
typedef short(__stdcall *lpInp32)(short);
typedef BOOL(__stdcall *lpIsInpOutDriverOpen)(void);

static lpOut32 gfpOut32;
static lpInp32 gfpInp32;
static lpIsInpOutDriverOpen gfpIsInpOutDriverOpen;
static HINSTANCE hInpOutDll;

static inline void outb(short byte, int address);
static inline short inb(int address);
#endif

static inline short SetLdn(short ldn);
static short OpenWatchdogLdn(void);
static inline void SetWatchdogConfig(short config);
static inline short GetWatchdogConfig(void);
static inline void SetWatchdogPme(short config);
static inline short GetWatchdogPme(void);

bool CheckLpcPermission(void) {
  //run _en, _dis lpc and set HAVE PERMS
  if(!EnableLpcAccess()){
    HAVE_PERMS = true;
  } else {
    HAVE_PERMS = false;
  }
  DisableLpcAccess();
  return HAVE_PERMS;
}

short EnableLpcAccess(void) {
#ifdef __MINGW32__
#ifdef __MINGW64__
  hInpOutDll = LoadLibrary("InpOutx64.dll");
#else
  hInpOutDll = LoadLibrary("InpOut32.dll");
#endif
  if (hInpOutDll == NULL) {
    return 1;
  }
  gfpOut32 = (lpOut32)GetProcAddress(hInpOutDll, "Out32");
  gfpInp32 = (lpInp32)GetProcAddress(hInpOutDll, "Inp32");
  gfpIsInpOutDriverOpen = (lpIsInpOutDriverOpen)GetProcAddress(hInpOutDll, "IsInpOutDriverOpen");

  if (!gfpIsInpOutDriverOpen()) {
    return 2;
  }
  return 0;
#else
  return ioperm(INDEX_REG, 2, 1);
#endif
}

void DisableLpcAccess(void) {
#ifdef __MINGW32__
  FreeLibrary(hInpOutDll);
#else
  ioperm(INDEX_REG, 2, 0);
#endif
}

short OpenLpc(void) {
  if (LPC_OPEN) {
    return 0;
  }
  if (!HAVE_PERMS) {
    if (!CheckLpcPermission()) {
      return -1;
    }
  }
  EnableLpcAccess();
  outb(KEY_ENTER, INDEX_REG);
  outb(KEY_ENTER, INDEX_REG);
  OLD_LDN = inb(DATA_REG);
  LPC_OPEN = true;
  return 0;
};
void CloseLpc(void) {
  SetLdn(OLD_LDN);
  OLD_LDN = 0;
  outb(KEY_EXIT, INDEX_REG);
  DisableLpcAccess();
  LPC_OPEN = false;
};

static inline short SetLdn(short ldn) {
  outb(LDN_SELECT, INDEX_REG);
  outb(ldn, DATA_REG);
  CUR_LDN = ldn;
}

short OpenDioLdn(void) {
  if (!LPC_OPEN && OpenLpc()) {
    return -1;
  }
  if (CUR_LDN == DIO_LDN) {
    return 0;
  }
  SetLdn(DIO_LDN);
  CUR_LDN = DIO_LDN;
  return 0;
};

short DioReadInputByte(void) {
  if (OpenDioLdn()) {
    return -1;
  }
  outb(DI_REG, INDEX_REG);
  return inb(DATA_REG);
}
short DioSetOutputByte(short byte) {
  if (OpenDioLdn()) {
    return -1;
  }
  outb(DO_REG, INDEX_REG);
  outb(byte, DATA_REG);
  return 0;
}

static short OpenWatchdogLdn(void) {
  if (!LPC_OPEN && OpenLpc()) {
    return -1;
  }
  if (CUR_LDN == WDT_LDN) {
    return 0;
  }
  SetLdn(WDT_LDN);
  CUR_LDN = WDT_LDN;
  return 0;
};

short EnableWatchdog(short time) {
  if(OpenWatchdogLdn()) {
    return -1;
  }
  SetWatchdogTime(time);
  SetWatchdogConfig(GetWatchdogConfig() & WDTMOUT_STS);
  SetWatchdogConfig(WD_EN | WD_PULSE | WD_PSWIDTH2);
  SetWatchdogPme(GetWatchdogPme() | WDOUT_EN);
  CloseLpc();
};

short DisableWatchdog(void) {
  if(OpenWatchdogLdn()) {
    return -1;
  }
  SetWatchdogConfig(GetWatchdogConfig() & ~WD_EN);
  SetWatchdogPme(GetWatchdogPme() & ~WDOUT_EN);
}
static inline void SetWatchdogConfig(short config) {
  outb(WDT_CONF_REG1, INDEX_REG);
  outb(config, DATA_REG);
}
static inline short GetWatchdogConfig(void) {
  outb(WDT_CONF_REG1, INDEX_REG);
  return inb(DATA_REG);
}
static inline void SetWatchdogPme(short config) {
  outb(WDT_PME_REG1, INDEX_REG);
  outb(config, DATA_REG);
}
static inline short GetWatchdogPme(void) {
  outb(WDT_PME_REG1, INDEX_REG);
  return inb(DATA_REG);
}

short SetWatchdogTime(short timeout) {
  if(OpenWatchdogLdn()) {
    return -1;
  }
  outb(WDT_TIME_REG, INDEX_REG);
  outb(timeout, DATA_REG);
}
short GetWatchdogTime(void) {
  if(OpenWatchdogLdn()) {
    return -1;
  }
  outb(WDT_TIME_REG, INDEX_REG);
  return inb(DATA_REG);
}

#ifdef __MINGW32__
static inline void outb(short byte, int address) {
  gfpOut32(address, byte);
}
static inline short inb(int address) {
  return gfpInp32(address);
}
#endif
