#ifndef F81866A_H
#define F81866A_H

#ifdef __MINGW32__
#include "windows.h"
#else
#include "sys/io.h"
#endif
#include "stdbool.h"

#define INDEX_REG       0x4E
#define DATA_REG	0x4F
#define LDN_SELECT      0x07
#define KEY_ENTER       0x87
#define KEY_EXIT        0xAA

#define DIO_LDN         0x06
#define DO_REG          0x89
#define DI_REG          0x82

#define WDT_LDN         0x07
#define WDT_TIME_REG    0xF6
#define WDT_CONF_REG1   0xF5
#define WDT_PME_REG1    0xFA

// WDT Configuration Reg settings
// WDTMOUT_STS: TIMEOUT flag, set when WDT_TIME_REG hits 0
// write '1' to clear flag
#define WDTMOUT_STS     0x40
// WD_EN: 1 Enable / 0 Disable watchdog timer
#define WD_EN           0x20
// WD_PULSE: 1 Pulse / 0 Level 
// Signal type when watchdog is reset (pulse to active state, or hold at active)
#define WD_PULSE        0x10
// WD_UNIT: 1 Seconds / 0 Minutes
// Timeout occurs after WDT_TIME_REG seconds or minutes
#define WD_UNIT         0x08
// WD_HACTIVE: 1 High / 0 Low
// Reset signal is active-high or active-low
#define WD_HACTIVE      0x04
// WD_PSWIDTH: length of reset pulse, if WD_PULSE is set
// 3 5 sec / 2 125 msec / 1 25 msec / 0 1 msec
#define WD_PSWIDTH3     0x03
#define WD_PSWIDTH2     0x02
#define WD_PSWIDTH1     0x01

// WDT PME Configuration Reg settings
// WDT_PME: PME flag, 1 if WDT PME event has occurred
#define WDT_PME         0x80
// WDT_PME_EN: Enable WDT PME (wakeup signal on WD timeout)
// 1 Enabled / 0 Disabled
#define WDT_PME_EN      0x40
// WDOUT_EN: enable reset signal output
// 1 Enabled / 0 Disabled
#define WDOUT_EN        0x01

bool CheckLpcPermission(void);
short EnableLpcAccess(void);
void DisableLpcAccess(void);
short OpenLpc(void);
void CloseLpc(void);
short OpenDioLdn(void);
short DioReadInputByte(void);
short DioSetOutputByte(short);
//short OpenWatchdogLdn(void);
short EnableWatchdog(short);
short DisableWatchdog(void);
short SetWatchdogTime(short);
short GetWatchdogTime(void);
#endif
