/***************************************************
*  CINCOZEDIO
*  Test/Demo for DIO control on Cincoze DS/DC/DE
*  series systems
****************************************************

Uses Linux built-in functions inb and outb to read/write to IO port addresses.

To use, open a terminal and navigate to this folder, then run "make"
to build the example. Process needs to run with permission to access IO ports - you can 
run with "sudo ./CincozeDIO" from the same terminal window or grant the user account 
appropriate permissions. 
When executed with no arguments, program will print the status 
of inputs and mirror them to outputs (DI0 -> DO0, etc). When executed with a single hex 
byte as an argument, will print input status and write hex byte to output.

When reading/writing (though this demo handles parsing the read byte automatically),
each bit corresponds to a single digital pin (documented below). Each bit will read 1
for an "un-triggered" pin and 0 for a "triggered" one.
Ex: ./CincozeDIO 0xfe
    Result: DIO 0 ON, DIO 1-3 OFF
****************************************************/

#include "f81866a.h"
#include "stdio.h"
#include "stdlib.h"
#include "unistd.h"

void test_wdt(short timeout) {
  int i = 0;

  EnableWatchdog(timeout);
  //SetWatchdogTime(timeout);

  printf("Rebooting in %d ", timeout);
  for(i = timeout+1; i > 0; i--) {
    printf("%d ", i-1);
#ifdef __MINGW32__
    Sleep(1000);
#else
    usleep(1000000);
#endif
  }
  //we never reach this point
  DisableWatchdog();
}

void test_dio(short do_read, short do_write, short out_val) {
  short in_val = 0;
  int i = 0;

  if (do_read) {
    in_val = DioReadInputByte();
    for (i = 3; i >= 0; i--) {
      printf("DI %d Value:   ", 3 - i);
      if(in_val & (128 >> i)) {
        printf("High\n");
      } else {
	printf(" Low\n");
      }
    }
    if (do_write && out_val < 0) {
      out_val = (~in_val >> 4) | 0xf0;
    }
  }

  if (out_val >= 0 && out_val <= 255) {
    DioSetOutputByte(out_val);
    printf("Write DO Value: %02X.\n", out_val);
  }

}


int menu(char* title, char* options[], int nOpts) {
  int i = 0;
  int sel = 0;
  printf("%s\n", title);
  for (i = 0; i < nOpts; i++) {
    printf(" %d. %s\n", i+1, options[i]);
  }
  printf("Enter selection: ");
  scanf("%d", &sel);
  return sel;
}

int main(int argc, char *argv[]) {
  // Opens the LPC port for communication
  // In Linux, this also obtains permissions for the LPC port
  // In Windows, this also loads the required dll
  char *menuopts[10];
  int val;

  if(!CheckLpcPermission()) {
    printf("Error opening LPC port. Check permissions and try again.\n");
  }

  menuopts[0] = "Digital IO";
  menuopts[1] = "Watchdog TImer";
  switch(menu("Cincoze LPC Function Sample", menuopts, 2)) {
  case 1:
    menuopts[0] = "Write input state to output";
    menuopts[1] = "Read input states";
    menuopts[2] = "Write value to output";
    switch(menu("Digital Input/Output Test", menuopts, 3)) {
    case 1:
      test_dio(1, 1, -1);
      break;
    case 2:
      test_dio(1, 0, -1);
      break;
    case 3:
      printf("Enter hex value for outputs: ");
      scanf("%2X", &val);
      test_dio(0, 1, (short)val);
      break;
    default:
      printf("Invalid selection.\n");
      return 1;
    }
    break;
  case 2:
    menuopts[0] = "Set watchdog timer and wait for reboot";
    switch(menu("Watchdog timer test", menuopts, 1)) {
    case 1:
      printf("Enter time to set watchdog: ");
      scanf("%d", &val);
      test_wdt((short)val);
      break;
    default:
      printf("Invalid selection.\n");
      return 1;
    }
    break;
  default:
    printf("Invalid selection.\n");
    return 1;
  }     
  printf("Press Enter to exit.");
  char anykey[10];
  scanf("%s", anykey);
  return 0;
}
